package com.wyattjmiller.repositories

import com.wyattjmiller.data.DatabaseManager
import com.wyattjmiller.data.DbRecipeTable
import com.wyattjmiller.entities.Recipe
import com.wyattjmiller.entities.RecipeDraft
import org.ktorm.dsl.delete
import org.ktorm.dsl.eq
import org.ktorm.dsl.insertAndGenerateKey
import org.ktorm.dsl.update
import org.ktorm.entity.*
import java.time.LocalDateTime

class RecipeDao : RecipeRepository {
    private val db = DatabaseManager()

    override fun getAllRecipies() : List<Recipe> {
        return db.database
            .sequenceOf(DbRecipeTable)
            .toList()
            .map {
                Recipe(it.id, it.name, it.desc, it.ingredients, it.createdTimestamp.toString(), it.author.id)
            }
    }

    override fun getRecipe(id: Int) : Recipe {
        return db.database
            .sequenceOf(DbRecipeTable)
            .firstOrNull {
                it.id eq id
            }
            .let {
                Recipe(it!!.id, it.name, it.desc, it.ingredients, it.createdTimestamp.toString(), it.author.id)
            }
    }

    override fun createRecipe(recipe: RecipeDraft) : Recipe {
        val key = db.database.insertAndGenerateKey(DbRecipeTable) {
            set(DbRecipeTable.name, recipe.name)
            set(DbRecipeTable.desc, recipe.desc)
            set(DbRecipeTable.ingredients, recipe.ingredients)
            set(DbRecipeTable.createdTimestamp, LocalDateTime.now())
            set(DbRecipeTable.author, recipe.author)
        } as Int

        return Recipe(key, recipe.name, recipe.desc, recipe.ingredients, LocalDateTime.now().toString(), recipe.author)
    }

    override fun updateRecipe(id: Int, recipe: RecipeDraft) : Boolean {
        val updatedRows = db.database.update(DbRecipeTable) {
            set(DbRecipeTable.name, recipe!!.name)
            set(DbRecipeTable.desc, recipe!!.desc)
            set(DbRecipeTable.ingredients, recipe!!.ingredients)
            where {
                it.id eq id
            }
        }

        return updatedRows > 0
    }

    override fun deleteRecipe(id: Int) : Boolean {
        val deletedRows = db.database.delete(DbRecipeTable) {
            it.id eq id
        }

        return deletedRows > 0
    }
}